unit Unit1; 

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, LResources, Forms, Controls, Graphics, Dialogs,
  ComCtrls, StdCtrls, ExtCtrls, LCLType,
  {$IFDEF UNIX}
    unix;
  {$ELSE}
    Windows;
  {$ENDIF}
type

  { TForm1 }

  TForm1 = class(TForm)
    ImageList1: TImageList;
    ListView1: TListView;
    Memo1: TMemo;
    TreeView1: TTreeView;
    procedure FormCreate(Sender: TObject);
    procedure ListView1DblClick(Sender: TObject);
    procedure TreeView1Change(Sender: TObject; Node: TTreeNode);
    procedure TreeView1Expanding(Sender: TObject; Node: TTreeNode;
                                 var AllowExpansion: Boolean);
    function Real_Directory(sname: string): boolean;
    procedure Show_Only_Dir(ParentNode: TTreeNode);
    procedure Show_in_ListView_Dir(ParentNode: TTreeNode);
    procedure Show_in_ListView_Files(ParentNode: TTreeNode);
    procedure Show_ListView_Dir(DirName: string);
    procedure Show_ListView_Files(DirName: string);
    {$IFDEF WINDOWS}
    function Find_Logical_Disks(): boolean;
    procedure SetAllDirectories;

    {$ENDIF}
  private
    { private declarations }
  public
    { public declarations }
  end; 

var
  Form1: TForm1; 
  path, oldpath: string;
implementation

{ TForm1 }

function TForm1.Real_Directory(sname: string): boolean;
begin
  result:= (sname <> '.') and (sname <> '..');
end;

{$IFDEF WINDOWS}
// Эти функции нужны только для Windows
function TForm1.Find_Logical_Disks(): boolean;
const
  Hard_Disk = 3;  // рассматриваем только жесткие диски
var
  size: LongWord;
  Drives: array[0..128] of char;
  pDrive: PChar;
  s: string;
begin
  size:= GetLogicalDriveStrings(SizeOf(Drives), Drives);
  if size = 0 then
  begin
    Result:= false;
    exit;
  end;
  if size > SizeOf(Drives) then
    raise Exception.Create(SysErrorMessage(ERROR_OUTOFMEMORY));
  pDrive:= Drives; // устанавливаем указатель на Drives
  while pDrive^ <> #0 do
  begin
    // если тип устройства жесткий диск
    if GetDriveType(pDrive) = Hard_Disk then
    begin
      s:= pDrive;
      s:= Copy(s, 1, 2);  // берем только имя раздела и двоеточие
      Memo1.Lines.Add(s); // добавляем имя раздела в Memo1
    end;
    inc(pDrive, 4);
  end;
end;

procedure TForm1.SetAllDirectories;
var
  i: integer;
  Node: TTreeNode;
begin
  TreeView1.BeginUpdate;
  for i:= 0 to Memo1.Lines.Count - 1 do
  begin
    Node:= TreeView1.Items.AddChild(nil, Memo1.Lines[i]);
    Node.ImageIndex:= 0;
    Node.SelectedIndex:= 0;
    Node.HasChildren:= true;
  end;
  TreeView1.EndUpdate;
end;
{$ENDIF}

procedure TForm1.FormCreate(Sender: TObject);
var
  Node: TTreeNode;
  srNode, srChild: TSearchRec;
  searchMask: string;
  SetDirWin: boolean = false;
begin
  Memo1.Clear;
  Memo1.Visible:= false;
  TreeView1.Images:= ImageList1;
  TreeView1.ExpandSignType:= tvestPlusMinus;
  TreeView1.BeginUpdate;
    {$IFDEF WINDOWS}
       // Определение логических дисков
      if Find_Logical_Disks() then
      begin
        SetAllDirectories;
        SetDirWin:= true;
      end
      else
      {Если произошла ошибка в функции Find_Logical_Disks(),
      то выбираем корневой каталог текущего диска}
        SetCurrentDir('\');
    {$ELSE}
      SetCurrentDir('/'); // корневой каталог в Linux
    {$ENDIF}
    if not SetDirWin then
    begin
      path:= GetCurrentDir;
      if FindFirst(path + '*', faDirectory, srNode) = 0 then
      begin
        repeat
          // Показываем только каталоги
          if (srNode.Attr and faDirectory <> 0)
          and (Real_Directory(srNode.Name)) then
          begin
            Node:= TreeView1.Items.AddChild(nil, SysToUTF8(srNode.Name));
            Node.ImageIndex:= 0;
            Node.SelectedIndex:= 0;
            {$IFDEF WINDOWS}
            searchMask:= path + srNode.Name + '\*';
            {$ELSE}
              searchMask:= path + srNode.Name + '/*';
            {$ENDIF}
            if FindFirst(searchMask, faDirectory, srChild) = 0 then
            repeat
              if (srChild.Attr and faDirectory <> 0) and Real_Directory(srChild.Name)
              then Node.HasChildren:= true;
            until (FindNext(srChild) <> 0) or Node.HasChildren;
            SysUtils.FindClose(srChild);
          end;
        until FindNext(srNode) <> 0;
        // Освобождение занятых ресурсов
        SysUtils.FindClose(srNode);
      end;
    end;
  TreeView1.EndUpdate;
  // установка свойств ListView1
  with ListView1 do
  begin
    Align:= alClient;
    ViewStyle:= vsReport;
    Columns.Add;
    Columns.Items[0].Width:= 425;
    ScrollBars:= ssAutoBoth;
    SmallImages:= ImageList1;
  end;
end;

procedure TForm1.Show_Only_Dir(ParentNode: TTreeNode);
var
  srNode, srChild: TSearchRec;
  Node: TTreeNode;
  searchMask: string;
begin
  Node:= ParentNode;
  path:= '';
  repeat
    {$IFDEF WINDOWS}
    path:= UTF8ToSys(Node.Text) + '\' + path;
    {$ELSE}
    path:= '/' + Node.Text + '/' + path;
    {$ENDIF}
    Node:= Node.Parent;
  until Node = nil;
  if FindFirst(path + '*', faDirectory, srNode) = 0 then
  repeat
  if (srNode.Attr and faDirectory <> 0) and Real_Directory(srNode.Name) then
  begin
    Node:= TreeView1.Items.AddChild(ParentNode, SysToUTF8(srNode.Name));
    Node.ImageIndex:= 0;
    Node.SelectedIndex:= 0;
    {$IFDEF WINDOWS}
      searchMask:= path + srNode.Name + '\*';
    {$ELSE}
      searchMask:= path + srNode.Name + '/*';
    {$ENDIF}
    if FindFirst(searchMask, faDirectory, srChild) = 0 then
    repeat
      if (srChild.Attr and faDirectory <> 0) and Real_Directory(srChild.Name)
      then Node.HasChildren:= true;
    until (FindNext(srChild) <> 0) or Node.HasChildren;
    SysUtils.FindClose(srChild);
  end;
  until FindNext(srNode) <> 0;
  SysUtils.FindClose(srNode);
end;

procedure TForm1.Show_in_ListView_Dir(ParentNode: TTreeNode);
var
  srNode, srChild: TSearchRec;
  Node: TTreeNode;
  ListItem: TListItem;
  searchMask: string;
begin
  Node:= ParentNode;
  path:= '';
  repeat
    {$IFDEF WINDOWS}
    path:= UTF8ToSys(Node.Text) + '\' + path;
    {$ELSE}
    path:= '/' + Node.Text + '/' + path;
    {$ENDIF}
    Node:= Node.Parent;
  until Node = nil;
  if FindFirst(path + '*', faDirectory, srNode) = 0 then
  repeat
  if (srNode.Attr and faDirectory <> 0) and Real_Directory(srNode.Name) then
  begin
    Node:= TreeView1.Items.AddChild(ParentNode, SysToUTF8(srNode.Name));
    if srNode.Attr and faHidden = 0 then
    begin
      ListItem:= ListView1.Items.Add;
      ListItem.Caption:= Node.Text;
      ListItem.ImageIndex:= 0;
    end;
    {$IFDEF WINDOWS}
      searchMask:= path + srNode.Name + '\*';
    {$ELSE}
      searchMask:= path + srNode.Name + '/*';
    {$ENDIF}
    if FindFirst(searchMask, faDirectory, srChild) = 0 then
    repeat
      if (srChild.Attr and faDirectory <> 0) and Real_Directory(srChild.Name)
      then Node.HasChildren:= true;
    until (FindNext(srChild) <> 0) or Node.HasChildren;
    SysUtils.FindClose(srChild);
  end;
  until FindNext(srNode) <> 0;
  SysUtils.FindClose(srNode);
end;

procedure TForm1.Show_in_ListView_Files(ParentNode: TTreeNode);
var
  srNode: TSearchRec;
  Node: TTreeNode;
  ListItem: TListItem;
begin
  Node:= ParentNode;
  path:= '';
  repeat
    {$IFDEF WINDOWS}
    path:= UTF8ToSys(Node.Text) + '\' + path;
    {$ELSE}
    path:= '/' + Node.Text + '/' + path;
    {$ENDIF}
    Node:= Node.Parent;
  until Node = nil;
  if FindFirst(path + '*', faAnyFile, srNode) = 0 then
  repeat
  if srNode.Attr and faDirectory = 0 then
  begin
    if srNode.Attr and faHidden = 0 then
    begin
      ListItem:= ListView1.Items.Add;
      ListItem.Caption:= SysToUTF8(srNode.Name);
      ListItem.ImageIndex:= 1;
    end;
  end;
  until FindNext(srNode) <> 0;
  SysUtils.FindClose(srNode);
end;

procedure TForm1.Show_ListView_Dir(DirName: string);
var
  srNode: TSearchRec;
  ListItem: TListItem;
begin
  oldpath:= path;
  {$IFDEF WINDOWS}
  path:= path + UTF8ToSys(DirName) + '\' ;
  {$ELSE}
  path:= '/' + path + DirName + '/';
  {$ENDIF}
  ListView1.Clear;
  if FindFirst(path + '*', faDirectory, srNode) = 0 then
  repeat
  if (srNode.Attr and faDirectory <> 0) and Real_Directory(srNode.Name) then
  begin
    ListItem:= ListView1.Items.Add;
    ListItem.Caption:= SysToUtf8(srNode.Name);
    ListItem.ImageIndex:= 0;
  end;
  until FindNext(srNode) <> 0;
  SysUtils.FindClose(srNode);
end;

procedure TForm1.Show_ListView_Files(DirName: string);
var
  srNode: TSearchRec;
  ListItem: TListItem;
begin
  path:= '';
  {$IFDEF WINDOWS}
  path:= oldpath + UTF8ToSys(DirName) + '\' ;
  {$ELSE}
  path:= '/' + oldpath + DirName + '/';
  {$ENDIF}
  if FindFirst(path + '*', faAnyFile, srNode) = 0 then
  repeat
  if srNode.Attr and faDirectory = 0 then
  begin
    if srNode.Attr and faHidden = 0 then
    begin
      ListItem:= ListView1.Items.Add;
      ListItem.Caption:= SysToUTF8(srNode.Name);;
      ListItem.ImageIndex:= 1;
    end;
  end;
  until FindNext(srNode) <> 0;
  SysUtils.FindClose(srNode);
end;

procedure TForm1.ListView1DblClick(Sender: TObject);
var
  s: string;
  DirName: string;
  srNode: TSearchRec;
begin
  if ListView1.Selected = nil then exit;
  {$IFDEF WINDOWS}
  s:= path + UTF8ToSys(ListView1.Selected.Caption);
  {$ELSE}
  s:= path + ListView1.Selected.Caption;
  {$ENDIF}
  if FindFirst(s, faAnyFile, srNode) = 0 then
  if (srNode.Attr and faDirectory) <> 0 then
  begin
    DirName:= ListView1.Selected.Caption;
    Show_ListView_Dir(DirName);
    Show_ListView_Files(DirName);
    SysUtils.FindClose(srNode);
  end;
end;

procedure TForm1.TreeView1Change(Sender: TObject; Node: TTreeNode);
begin
  if Node = nil then exit;
  TreeView1.BeginUpdate;
  ListView1.BeginUpdate;
  ListView1.Clear;
  Node.DeleteChildren;
  Show_in_ListView_Dir(Node);
  Show_in_ListView_Files(Node);
  Node.Expanded:= true;
  ListView1.EndUpdate;
  TreeView1.EndUpdate;
end;

procedure TForm1.TreeView1Expanding(Sender: TObject; Node: TTreeNode;
  var AllowExpansion: Boolean);
begin
  TreeView1.BeginUpdate;
  Node.DeleteChildren;
  Show_Only_Dir(Node);
  TreeView1.EndUpdate;
end;

initialization
  {$I unit1.lrs}

end.

